use Devel::AssertOS qw[-MicrosoftWindows];
# This Build.PL for App-migrate was generated by Dist::Zilla::Plugin::ModuleBuildTiny 0.010.
use strict;
use warnings;

use 5.010001;
use Module::Build::Tiny 0.039;
Build_PL();
